<?php

namespace TdySports\Table;

use TdySports\Model;
use TdyCommons\Table\Table;
use Zend\Db\ResultSet\HydratingResultSet;
use Zend\Db\Sql\Delete;
use Zend\Db\Sql\Insert;
use Zend\Db\Sql\Predicate;
use Zend\Db\Sql\Select;
use Zend\Db\Sql\Sql;
use Zend\Db\Sql\Update;
use Zend\Db\Sql\Where;
use Zend\Paginator\Adapter\DbSelect;
use Zend\Paginator\Paginator;

/**
 * Class Schedules
 *
 * @package TdySports\Table
 *
 * @author  James Lloyd Atwil <james@nadows.com>
 */
class Schedules extends Table
{

    /**
     * @param string $q
     * @param array  $exclusion
     * @param array  $filters
     * @param array  $sort
     * @param bool   $paginated
     *
     * @return bool|HydratingResultSet|Paginator
     */
    public function fetch($q = '', $exclusion = [], $filters = [], $sort = [], $paginated = false)
    {
        $this->getLogger()->debug($this->indent . 'Schedules::fetch() start.');

        $select = new Select();
        $select->from($this->getTableGateway()->getTable());
        $conditions = [];

        $where = new Where([]);
        $where->addPredicate(new Predicate\Expression('is_active = ?', 1));
        $conditions[] = $where;

        if (!empty($q)) {
            $where = new Where([], Predicate\PredicateSet::OP_OR);
            $where->addPredicate(new Predicate\Like($this->columns['name'], '%' . $q . '%'));
            $conditions[] = $where;
        }

        if (!empty($filters)) {
            $where = new Where([]);
            foreach ($filters as $k => $v) {
                if (is_array($v)) {
                    $where->addPredicate(new Predicate\In($k, $v));
                } else {
                    $where->addPredicate(new Predicate\Expression($k . ' = ?', $v));
                }
            }
            $conditions[] = $where;
        }

        if (!empty($exclusion)) {
            $where = new Where([]);
            $where->addPredicate(new Predicate\NotIn('id', $exclusion));
            $conditions[] = $where;
        }

        $select->where($conditions, Predicate\PredicateSet::OP_AND);

        if (!is_null($sort)) {
            if ($this->isSortEmpty($sort)) {
                $select->order(
                    [
                        $this->columns['id'] => 'ASC',
                    ]
                );
            } else {
                $select->order($sort);
            }
        } else {
            $select->order($sort);
        }

        $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $select->getSqlString($this->getAdapter()->getPlatform()));

        $resultSet = new HydratingResultSet();
        $resultSet->setObjectPrototype(new Model\Schedule());

        if ($paginated) {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Is Paginable: YES');

            $paginatorAdapter = new DbSelect($select, $this->getTableGateway()->getSql(), $resultSet);
            $paginator        = new Paginator($paginatorAdapter);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Returns Paginator instance.');
            $this->getLogger()->debug($this->indent . 'Schedules::fetch() end.');

            return $paginator;
        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Is Paginable: NO');

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($select);

            $results = $statement->execute();
            $resultSet->initialize($results);

            if (!$resultSet) {
                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Returns FALSE');
                $this->getLogger()->debug($this->indent . 'Schedules::fetch() end.');

                return false;
            } else {
                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Returns HydratingResultSet instance.');
                $this->getLogger()->debug($this->indent . 'Schedules::fetch() end.');

                return $resultSet;
            }
        }
    }

    /**
     * Returns the Sport.
     *
     * @param int $id
     *
     * @return Model\Schedule|null
     * @throws \Exception
     */
    public function get($id)
    {
        $rowset = $this->tableGateway->select([$this->columns['id'] => $id]);
        $row    = $rowset->current();
        if (!$row) {
            return null;
        }

        return $row;
    }

    /**
     * @param Model\Schedule $schedule
     *
     * @return Model\Schedule
     * @throws \Exception
     */
    public function save(Model\Schedule $schedule)
    {
        $this->getLogger()->debug($this->indent . 'Schedules::save() start.');

        $data = [
            $this->columns['slug']       => $schedule->slug,
            $this->columns['sport-id']   => $schedule->sportId,
            $this->columns['league-id']  => $schedule->leagueId,
            $this->columns['season-id']  => $schedule->seasonId,
            $this->columns['week-id']    => $schedule->weekId,
            $this->columns['name']       => $schedule->name,
            $this->columns['team-home']  => $schedule->teamHome,
            $this->columns['team-away']  => $schedule->teamAway,
            $this->columns['venue']      => $schedule->venue,
            $this->columns['status']     => $schedule->status,
            $this->columns['score-home'] => $schedule->scoreHome,
            $this->columns['score-away'] => $schedule->scoreAway,
            $this->columns['ending']     => $schedule->ending,
        ];

        $id = (int) $schedule->id;
        if ($id == 0) {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Inserting Schedule record.');

            $data['is_active']  = 1;
            $data['created_by'] = $schedule->createdBy;
            $data['created_on'] = date('Y-m-d H:i:s');

            $insert = new Insert($this->getTableGateway()->getTable());
            $insert->values($data);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $insert->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($insert);

            $statement->execute();

            $id = $this->getAdapter()->getDriver()->getLastGeneratedValue();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ' . $id . ' has been inserted.');
            $this->getLogger()->debug($this->indent . 'Schedules::save() end.');

            return $this->get($id);
        } else {
            if ($this->get($id)) {
                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Updating Schedule record.');

                $data[$this->columns['updated-by']] = $schedule->updatedBy;
                $data[$this->columns['updated-on']] = date('Y-m-d H:i:s');

                $update = new Update($this->getTableGateway()->getTable());
                $update->set($data);
                $update->where([$this->columns['id'] => $id]);

                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

                $sql       = new Sql($this->tableGateway->getAdapter());
                $statement = $sql->prepareStatementForSqlObject($update);

                $statement->execute();

                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ' . $id . ' has been updated.');
                $this->getLogger()->debug($this->indent . 'Schedules::save() end.');

                return $this->get($id);
            } else {
                $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ID ' . $id . ' does not exists. Update failed.');
                $this->getLogger()->debug($this->indent . 'Schedules::save() end.');

                throw new \Exception('Sport ID does not exist');
            }
        }
    }

    /**
     * @param Model\Schedule $schedule
     *
     * @return null|Model\Schedule
     * @throws \Exception
     */
    public function updateScores(Model\Schedule $schedule)
    {
        $this->getLogger()->debug($this->indent . 'Schedules::updateScores() start.');

        $data = [
            $this->columns['score-home'] => $schedule->scoreHome,
            $this->columns['score-away'] => $schedule->scoreAway,
        ];

        if ($this->get($schedule->id)) {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Updating Schedule record.');

            $data[$this->columns['updated-by']] = $schedule->updatedBy;
            $data[$this->columns['updated-on']] = date('Y-m-d H:i:s');

            $update = new Update($this->getTableGateway()->getTable());
            $update->set($data);
            $update->where([$this->columns['id'] => $schedule->id]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($update);

            $statement->execute();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ' . $schedule->id . ' scores has been updated.');
            $this->getLogger()->debug($this->indent . 'Schedules::updateScores() end.');

            return $this->get($schedule->id);
        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ID ' . $schedule->id . ' does not exists. Update failed.');
            $this->getLogger()->debug($this->indent . 'Schedules::updateScores() end.');

            throw new \Exception('Schedule ID does not exist');
        }
    }

    /**
     * @param Model\Schedule $schedule
     *
     * @throws \Exception
     */
    public function delete(Model\Schedule $schedule)
    {
        $this->getLogger()->debug($this->indent . 'Schedules::delete() start.');

        if ($this->get($schedule->id)) {
            $data = [
                $this->columns['is-active']  => 0,
                $this->columns['deleted-by'] => $schedule->deletedBy,
                $this->columns['deleted-on'] => date('Y-m-d H:i:s'),
            ];

            $update = new Update($this->getTableGateway()->getTable());
            $update->set($data);
            $update->where([$this->columns['id'] => $schedule->id]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($update);

            $statement->execute();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ' . $schedule->id . ' has been deleted.');
            $this->getLogger()->debug($this->indent . 'Schedules::delete() end.');

        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ID ' . $schedule->id . ' does not exists. Deletion failed.');
            $this->getLogger()->debug($this->indent . 'Schedules::delete() end.');

            throw new \Exception('Sport ID does not exist');
        }
    }

    /**
     * Restore Sport to the table. Sets the Sport to active.
     *
     * @param Model\Schedule $schedule
     *
     * @throws \Exception
     */
    public function restore(Model\Schedule $schedule)
    {
        $this->getLogger()->debug($this->indent . 'Schedules::restore() start.');

        if ($this->get($schedule->id)) {
            $data = [
                $this->columns['is-active']  => 1,
                $this->columns['updated-by'] => $schedule->updatedBy,
                $this->columns['updated-on'] => date('Y-m-d H:i:s'),
            ];

            $update = new Update($this->getTableGateway()->getTable());
            $update->set($data);
            $update->where([$this->columns['id'] => $schedule->id]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($update);

            $statement->execute();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ' . $schedule->id . ' has been restored.');
            $this->getLogger()->debug($this->indent . 'Schedules::restore() end.');

        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ID ' . $schedule->id . ' does not exists. Restoration failed.');
            $this->getLogger()->debug($this->indent . 'Schedules::restore() end.');

            throw new \Exception('Schedule ID does not exist');
        }
    }

    /**
     * Permanently delete the Schedule.
     *
     * @param Model\Schedule $schedule
     *
     * @throws \Exception
     */
    public function purge(Model\Schedule $schedule)
    {
        $this->getLogger()->debug($this->indent . 'Schedules::purge() start.');

        if ($this->get($schedule->id)) {
            $update = new Delete($this->getTableGateway()->getTable());
            $update->where([$this->columns['id'] => $schedule->id]);

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'SQL Statement: ' . $update->getSqlString($this->getAdapter()->getPlatform()));

            $sql       = new Sql($this->tableGateway->getAdapter());
            $statement = $sql->prepareStatementForSqlObject($update);

            $statement->execute();

            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ' . $schedule->id . ' has been purged, as in permanently deleted.');
            $this->getLogger()->debug($this->indent . 'Schedules::purge() end.');

        } else {
            $this->getLogger()->debug($this->indent . self::LOGGER_INDENT . 'Schedule ID ' . $schedule->id . ' does not exists. Purge failed.');
            $this->getLogger()->debug($this->indent . 'Schedules::purge() end.');

            throw new \Exception('Schedule ID does not exist');
        }

    }

}
